
const fs = require('fs');
const path = require('path');

const version = "0.1.54";

const wasmBuffer = fs.readFileSync(path.join(__dirname, `brickwasm-${version}.wasm`));

const __state = new Promise((resolve, reject) => {
    WebAssembly.instantiate(wasmBuffer).then((obj) => {
        const state = {
            instance: obj.instance,
            version_major: obj.instance.exports.version_major(),
            version_minor: obj.instance.exports.version_minor(),
            version_patch: obj.instance.exports.version_patch(),
            allocate_context: obj.instance.exports.allocate_context,
            deallocate_context: obj.instance.exports.deallocate_context,
            allocate_string: obj.instance.exports.allocate_string,
            deallocate_string: obj.instance.exports.deallocate_string,
            tokenize: obj.instance.exports.tokenize,
            parse: obj.instance.exports.parse,
            store: obj.instance.exports.store,
            classify_tokens: obj.instance.exports.classify_tokens,
            bind: obj.instance.exports.brick_bind,
            classify_bound_tokens: obj.instance.exports.classify_bound_tokens,
            query_documents: obj.instance.exports.query_documents,
            query_document: obj.instance.exports.query_document,
            query_document_debug: obj.instance.exports.query_document_debug,
            query_model_names: obj.instance.exports.query_model_names,
            trigger_panic: obj.instance.exports.trigger_panic
        };
        if (`${state.version_major}.${state.version_minor}.${state.version_patch}` === version) {
            resolve(state);
        } else {
            reject("Version mismatch, wasm file corrupt?");
        }
    }).catch((err) => reject(err));
});

let __resolved_state = undefined;

function allocateString(state, str) {
    const encoder = new TextEncoder();
    const bytes = encoder.encode(str);
    const ptr = state.allocate_string(bytes.length);
    const memory = new Uint8Array(state.instance.exports.memory.buffer);
    memory.subarray(ptr, ptr + bytes.length).set(bytes);
    return { ptr, size: bytes.length };
}

function decodeFromOutputMemory(state, ptr) {
    const memory = new Uint8Array(state.instance.exports.memory.buffer);
    const decoder = new TextDecoder();
    const lengthString = decoder.decode(memory.subarray(ptr, ptr + 8));
    const length = parseInt(lengthString);
    const decoded = decoder.decode(memory.subarray(ptr + 8, ptr + length));
    state.deallocate_string(ptr, length);
    return decoded;
}

async function init() {
    __resolved_state = await __state;
}

const tokenize = (source, uri) => {
    if (__resolved_state === undefined) {
        return { tokens: [], errors: [ { error_code: 'ModuleNotInitialized', line: 0, column: 0, width: 0 }]}
    }
    const source_string = allocateString(__resolved_state, source);
    const uri_string = allocateString(__resolved_state, uri || "tokenize");
    const output_ptr = __resolved_state.tokenize(source_string.ptr, source_string.size, uri_string.ptr, uri_string.size);
    const json = decodeFromOutputMemory(__resolved_state, output_ptr);
    try {
        return JSON.parse(json);
    } catch {
        return { tokens: [], errors: [ { error_code: 'InvalidJson', line: 0, column: 0, width: 0 }] };
    }
};

const parse = (source, uri, namespace) => {
    if (__resolved_state === undefined) {
        return { document: null, errors: [ { error_code: 'ModuleNotInitialized', line: 0, column: 0, width: 0 }]}
    }
    const source_string = allocateString(__resolved_state, source);
    const uri_string = allocateString(__resolved_state, uri || "parse");
    const namespace_string = allocateString(__resolved_state, namespace || "");
    const output_ptr = __resolved_state.parse(
        source_string.ptr, source_string.size,
        uri_string.ptr, uri_string.size,
        namespace_string.ptr, namespace_string.size);
    const json = decodeFromOutputMemory(__resolved_state, output_ptr);
    try {
        return JSON.parse(json);
    } catch {
        return { document: undefined, errors: [ { error_code: 'InvalidJson', line: 0, column: 0, width: 0 }] };
    }
};

const classify_tokens = (source, uri) => {
    if (__resolved_state === undefined) {
        return { tokens: [], errors: [ { error_code: 'ModuleNotInitialized', line: 0, column: 0, width: 0 }]}
    }
    const source_string = allocateString(__resolved_state, source);
    const uri_string = allocateString(__resolved_state, uri || "classify");
    const output_ptr = __resolved_state.classify_tokens(source_string.ptr, source_string.size, uri_string.ptr, uri_string.size);
    const json = decodeFromOutputMemory(__resolved_state, output_ptr);
    try {
        return JSON.parse(json);
    } catch {
        console.error("Invalid json", json);
        return { tokens: [], errors: [ { error_code: 'InvalidJson', line: 0, column: 0, width: 0 }] };
    }
};

const new_context = () => {
    if (__resolved_state === undefined) {
        return undefined;
    }
    return __resolved_state.allocate_context();
}

const destroy_context = (context) => {
    if (__resolved_state === undefined) {
        return;
    }
    __resolved_state.deallocate_context(context);
}

const store = (context, source, uri, namespace) => {
    if (__resolved_state === undefined) {
        return { document: null, errors: [ { error_code: 'ModuleNotInitialized', line: 0, column: 0, width: 0 }]}
    }
    const source_string = allocateString(__resolved_state, source);
    const uri_string = allocateString(__resolved_state, uri || "store");
    const namespace_string = allocateString(__resolved_state, namespace || "");
    const output_ptr = __resolved_state.store(
        context,
        source_string.ptr, source_string.size,
        uri_string.ptr, uri_string.size,
        namespace_string.ptr, namespace_string.size);
    const json = decodeFromOutputMemory(__resolved_state, output_ptr);
    try {
        return JSON.parse(json);
    } catch {
        return { document: undefined, errors: [ { error_code: 'InvalidJson', line: 0, column: 0, width: 0 }] };
    }
};

const bind = (context) => {
    if (__resolved_state === undefined) {
        return { errors: [ { error_code: 'ModuleNotInitialized', line: 0, column: 0, width: 0 }]}
    }
    const output_ptr = __resolved_state.bind(context);
    const json = decodeFromOutputMemory(__resolved_state, output_ptr);
    try {
        return JSON.parse(json);
    } catch {
        return { errors: [ { error_code: 'InvalidJson', line: 0, column: 0, width: 0 }] };
    }
};

const classify_bound_tokens = (context, uri) => {
    if (__resolved_state === undefined) {
        return { tokens: [], errors: [ { error_code: 'ModuleNotInitialized', line: 0, column: 0, width: 0 }]}
    }
    if (uri === null || uri === undefined) {
        return { tokens: [], errors: [ { error_code: 'MissingUri', line: 0, column: 0, width: 0 }]}
    }
    const uri_string = allocateString(__resolved_state, uri);
    const output_ptr = __resolved_state.classify_bound_tokens(context, uri_string.ptr, uri_string.size);
    const json = decodeFromOutputMemory(__resolved_state, output_ptr);
    try {
        return JSON.parse(json);
    } catch {
        return { tokens: [], errors: [ { error_code: 'InvalidJson', line: 0, column: 0, width: 0 }] };
    }
};

const query_documents = (context) => {
    if (__resolved_state === undefined) {
        return { tokens: [], errors: [ { error_code: 'ModuleNotInitialized', line: 0, column: 0, width: 0 }]}
    }
    const output_ptr = __resolved_state.query_documents(context);
    const json = decodeFromOutputMemory(__resolved_state, output_ptr);
    try {
        return JSON.parse(json);
    } catch {
        return { tokens: [], errors: [ { error_code: 'InvalidJson', line: 0, column: 0, width: 0 }] };
    }
};

const query_document = (context, uri) => {
    if (__resolved_state === undefined) {
        return { document: null, errors: [ { error_code: 'ModuleNotInitialized', line: 0, column: 0, width: 0 }]}
    }
    const uri_string = allocateString(__resolved_state, uri);
    const output_ptr = __resolved_state.query_document(context, uri_string.ptr, uri_string.size);
    const json = decodeFromOutputMemory(__resolved_state, output_ptr);
    try {
        return JSON.parse(json);
    } catch {
        return { document: null, errors: [ { error_code: 'InvalidJson', line: 0, column: 0, width: 0 }] };
    }
};

const query_document_debug = (context, uri) => {
    if (__resolved_state === undefined) {
        return { document: null, errors: [ { error_code: 'ModuleNotInitialized', line: 0, column: 0, width: 0 }]}
    }
    const uri_string = allocateString(__resolved_state, uri);
    const output_ptr = __resolved_state.query_document_debug(context, uri_string.ptr, uri_string.size);
    const json = decodeFromOutputMemory(__resolved_state, output_ptr);
    try {
        return JSON.parse(json);
    } catch {
        return { document: null, errors: [ { error_code: 'InvalidJson', line: 0, column: 0, width: 0 }] };
    }
};

const query_model_names = (context) => {
    if (__resolved_state === undefined) {
        return { model_names: [], errors: [ { error_code: 'ModuleNotInitialized', line: 0, column: 0, width: 0 }]}
    }
    const output_ptr = __resolved_state.query_model_names(context);
    const json = decodeFromOutputMemory(__resolved_state, output_ptr);
    try {
        return JSON.parse(json);
    } catch {
        return { model_names: [], errors: [ { error_code: 'InvalidJson', line: 0, column: 0, width: 0 }] };
    }
};

const trigger_panic = () => {
    if (__resolved_state === undefined) {
        return { value: 0, error: 'ModuleNotInitialized' };
    }
    const output_ptr = __resolved_state.trigger_panic();
    const json = decodeFromOutputMemory(__resolved_state, output_ptr);
    try {
        return JSON.parse(json);
    } catch {
        return { value: 0, error: 'Invalid json: ' + json };
    }
};

module.exports = { init, tokenize, parse, classify_tokens, new_context, destroy_context, store, bind, classify_bound_tokens, query_documents, query_document, query_document_debug, query_model_names, trigger_panic };
