"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isFileInsideDirectory = exports.findBundleFrom = exports.loadAllBundlesInPath = void 0;
const fs = require("fs");
const path = require("path");
function extractBundleName(configFilePath) {
    const source = fs.readFileSync(configFilePath, 'utf8').toString();
    const match = source.match(/name:\s*"(.*)"/);
    return match ? match[1] : undefined;
}
function findConfigFiles(bundlePath, depth = 0) {
    const files = fs.readdirSync(bundlePath);
    const configFiles = [];
    for (const file of files) {
        let filepath = path.join(bundlePath, file);
        let stat = fs.statSync(filepath);
        if (stat.isDirectory() && depth === 0) {
            Array.prototype.push.apply(configFiles, findConfigFiles(filepath, depth + 1));
        }
        else if (file === 'config.brick' || file === 'config.openplx') {
            configFiles.push(filepath);
        }
    }
    return configFiles;
}
function findBrickFiles(bundle_path, current_path) {
    const files = fs.readdirSync(current_path);
    const brickFiles = [];
    for (const file of files) {
        let filepath = path.join(current_path, file);
        let stat = fs.statSync(filepath);
        if (stat.isDirectory()) {
            Array.prototype.push.apply(brickFiles, findBrickFiles(bundle_path, filepath));
        }
        else if ((file.endsWith('.brick') || file.endsWith('.openplx')) && file !== 'config.brick' && file !== 'config.openplx') {
            brickFiles.push(path.relative(bundle_path, filepath));
        }
    }
    return brickFiles;
}
function loadAllBundlesInPath(bundlePath) {
    const config_file_paths = findConfigFiles(bundlePath);
    const names = config_file_paths.map(config_path => extractBundleName(config_path));
    const bundles = [];
    for (let i = 0; i < config_file_paths.length; i++) {
        if (names[i]) {
            const root_path = path.dirname(config_file_paths[i]);
            bundles.push({
                name: names[i],
                root_path,
                files: findBrickFiles(root_path, root_path)
            });
        }
    }
    return bundles;
}
exports.loadAllBundlesInPath = loadAllBundlesInPath;
function findBrickConfigInDirectory(directory) {
    if (!fs.existsSync(directory)) {
        console.error(`Directory not found ${directory}, ignoring...`);
        return null;
    }
    const potential_brick_config = path.join(directory, "config.brick");
    if (fs.existsSync(potential_brick_config)) {
        return potential_brick_config;
    }
    const potential_openplx_config = path.join(directory, "config.openplx");
    if (fs.existsSync(potential_openplx_config)) {
        return potential_openplx_config;
    }
    return null;
}
function findBrickConfigFromDirectory(directory) {
    const direct_path = findBrickConfigInDirectory(directory);
    if (direct_path !== null) {
        return direct_path;
    }
    if (path.basename(directory) !== directory) {
        return findBrickConfigFromDirectory(path.basename(directory));
    }
    return null;
}
function findBundleFrom(startingPoint) {
    let config_file_path = null;
    const lstat = fs.lstatSync(startingPoint);
    if (lstat.isDirectory()) {
        config_file_path = findBrickConfigFromDirectory(startingPoint);
    }
    else if (lstat.isFile()) {
        if (path.basename(startingPoint) === 'config.brick'
            || path.basename(startingPoint) == 'config.openplx') {
            config_file_path = startingPoint;
        }
        else {
            config_file_path = findBrickConfigFromDirectory(path.dirname(startingPoint));
        }
    }
    else {
        config_file_path = findBrickConfigFromDirectory(path.dirname(startingPoint));
    }
    if (config_file_path !== null) {
        const name = extractBundleName(config_file_path);
        if (name) {
            const root_path = path.dirname(config_file_path);
            return {
                name,
                root_path,
                files: findBrickFiles(root_path, root_path)
            };
        }
    }
    return null;
}
exports.findBundleFrom = findBundleFrom;
function isFileInsideDirectory(directoryPath, filePath) {
    const relativePath = path.relative(directoryPath, filePath);
    return !relativePath.startsWith('..') && !path.isAbsolute(relativePath);
}
exports.isFileInsideDirectory = isFileInsideDirectory;
//# sourceMappingURL=bundle-loader.js.map