"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BrickCompletionItemProvider = void 0;
const vscode = require("vscode");
const path = require("path");
const fs = require("fs");
const brickwasm = require("@algoryx/brickwasm");
class BrickCompletionItemProvider {
    constructor(context) {
        this.context = context;
    }
    provideMatchingTypes(position, model_names, current_ns, target, primitiveTypes) {
        const completionItems = [];
        const targetRange = new vscode.Range(position.translate(0, -target.length), position);
        // Find all model declarations that match
        for (const model_name of model_names) {
            if (model_name.toLocaleLowerCase().includes(target.toLocaleLowerCase())) {
                const item = new vscode.CompletionItem(model_name);
                item.kind = vscode.CompletionItemKind.Class;
                item.range = targetRange;
                item.insertText = model_name;
                item.sortText = model_name;
                item.filterText = target;
                completionItems.push(item);
            }
        }
        // Find all primitive types that match
        /*for (const primitiveType of primitiveTypes) {
            if (primitiveType.includes(target)) {
                completionItems.push({
                    label: primitiveType,
                    insertText: primitiveType,
                    range: { inserting: targetRange, replacing: targetRange }
                });
            }
        }*/
        if (completionItems.length === 0) {
            return [new vscode.CompletionItem('')];
        }
        return completionItems;
    }
    provideCompletionItems(textDocument, position, token, context) {
        const currentLine = textDocument.lineAt(position.line).text;
        /**
         * Since autocomplete is usually used on a line that is not valid brick syntax the parser will be of no use,
         * instead we rely on complex, best effort, regular expressions to interpret the brick syntax on the current line.
         */
        const parent_re = new RegExp('^[a-zA-Z0-9]+ is (?<target>[a-zA-Z0-9_\\.]*):?$');
        const var_decl_type_re = new RegExp('^[ \\t]+[a-zA-Z0-9_]+ (?:is|becomes) (?<target>[a-zA-Z0-9_\\.]*):?[^:]*$');
        const return_type_re = new RegExp('^\\s*(?:static )?(?:fn|operator) [a-zA-Z0-9_+*\\-/]+\\([\\s:a-zA-Z0-9_,.]*\\) -> (?<target>[a-zA-Z0-9_\\.]*)$');
        const uri = fs.realpathSync.native(path.normalize(textDocument.uri.fsPath));
        const query_document_result = brickwasm.query_document(this.context, uri);
        const { model_names, errors } = brickwasm.query_model_names(this.context);
        const current_ns = query_document_result.document ? query_document_result.document.namespace : [];
        if (errors.length > 0) {
            return [new vscode.CompletionItem('')];
        }
        const primitive_types = ['Real', 'String', 'Int', 'Bool'];
        if (parent_re.test(currentLine)) {
            const matches = currentLine.match(parent_re);
            if (matches === null || matches.groups === undefined)
                return [];
            const target = matches.groups['target'];
            return this.provideMatchingTypes(position, model_names, current_ns, target, []);
        }
        if (return_type_re.test(currentLine)) {
            const matches = currentLine.match(return_type_re);
            if (matches === null || matches.groups === undefined)
                return [];
            const target = matches.groups['target'];
            return this.provideMatchingTypes(position, model_names, current_ns, target, primitive_types);
        }
        else if (var_decl_type_re.test(currentLine)) {
            const matches = currentLine.match(var_decl_type_re);
            if (matches === null || matches.groups === undefined)
                return [];
            const target = matches.groups['target'];
            return this.provideMatchingTypes(position, model_names, current_ns, target, primitive_types);
        }
        return [new vscode.CompletionItem('')];
    }
    resolveCompletionItem(item, token) {
        return item;
    }
}
exports.BrickCompletionItemProvider = BrickCompletionItemProvider;
//# sourceMappingURL=completionitem-provider.js.map