"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.deactivate = exports.activate = void 0;
const vscode = require("vscode");
const fs = require("fs");
const path = require("path");
const brickwasm = require("@algoryx/brickwasm");
const token_provider_1 = require("./token_provider");
const definitions_provider_1 = require("./definitions-provider");
const bundle_loader_1 = require("./bundle-loader");
const completionitem_provider_1 = require("./completionitem-provider");
const brickimportcontent_provider_1 = require("./brickimportcontent-provider");
const node_1 = require("vscode-languageclient/node");
const tokenTypes = ['class', 'method', 'keyword', 'type', 'string', 'number', 'property', 'operator', 'parameter', 'variable'];
const tokenModifiers = ['declaration', 'modification', 'definition', 'readonly'];
const legend = new vscode.SemanticTokensLegend(tokenTypes, tokenModifiers);
let brick_context = undefined;
let client = undefined;
function loadBundles(bundle_path) {
    const bundles = (0, bundle_loader_1.loadAllBundlesInPath)(bundle_path);
    for (const bundle of bundles) {
        for (const file of bundle.files) {
            const file_path = path.normalize(bundle.root_path + '/' + file);
            const source = fs.readFileSync(file_path, 'utf8').toString();
            const namespace = [bundle.name, ...file.split(path.sep)];
            namespace.pop();
            const { errors } = brickwasm.store(brick_context, source, fs.realpathSync.native(file_path), namespace.join("."));
            if (errors.length > 0) {
                vscode.window.showErrorMessage(`Unable to parse ${file}`);
            }
        }
    }
}
function loadLocalBundleIfNotInPath(bundle_path, document) {
    if (!(0, bundle_loader_1.isFileInsideDirectory)(bundle_path, document.uri.fsPath)) {
        const bundle = (0, bundle_loader_1.findBundleFrom)(document.uri.fsPath);
        if (bundle !== null) {
            for (const file of bundle.files) {
                const file_path = path.normalize(bundle.root_path + '/' + file);
                const source = fs.readFileSync(file_path, 'utf8').toString();
                const namespace = [bundle.name, ...file.split(path.sep)];
                namespace.pop();
                const { errors } = brickwasm.store(brick_context, source, fs.realpathSync.native(file_path), namespace.join("."));
                if (errors.length > 0) {
                    vscode.window.showErrorMessage(`Unable to parse ${file}`);
                }
            }
        }
    }
}
function activateLanguageClient(config) {
    const server_cmd = config.get("languageServerCommand");
    const args = server_cmd.split(' ');
    const cmd = args[0];
    args.shift();
    const serverOptions = { command: cmd, args: args, transport: node_1.TransportKind.stdio };
    // Options to control the language client
    const clientOptions = {
        documentSelector: [{ pattern: '*.openplx', language: 'openplx' }, { pattern: '*.brick', language: 'openplx' }]
    };
    // Create the language client and start the client.
    client = new node_1.LanguageClient('brick_lsp', 'BRICK LSP', serverOptions, clientOptions);
    // Start the client. This will also launch the server
    client.start();
}
// This method is called when your extension is activated
// Your extension is activated the very first time the command is executed
async function activate(context) {
    const brick_config = vscode.workspace.getConfiguration('openplx');
    const enable_lsp = brick_config.get("enableLSP");
    const outputChannel = vscode.window.createOutputChannel('OpenPLX language extension');
    if (enable_lsp) {
        activateLanguageClient(brick_config);
    }
    else {
        await brickwasm.init();
        brick_context = brickwasm.new_context();
        const selector = 'openplx';
        const errors = vscode.languages.createDiagnosticCollection('errors');
        const token_provider = new token_provider_1.TokenProvider(legend, brick_context, outputChannel, errors);
        const definition_provider = new definitions_provider_1.DefinitionProvider(brick_context);
        const bundle_path = brick_config.get('bundlePath');
        const anytobrick_command = brick_config.get('anytoopenplxCommand');
        if (!bundle_path) {
            vscode.window.showWarningMessage("Brick: Bundle path not set, check your settings");
        }
        loadBundles(bundle_path);
        vscode.languages.registerDocumentSemanticTokensProvider(selector, token_provider, legend);
        if (vscode.window.activeTextEditor) {
            if (vscode.window.activeTextEditor.document.fileName.endsWith('.brick')
                || vscode.window.activeTextEditor.document.fileName.endsWith('.openplx')) {
                loadLocalBundleIfNotInPath(bundle_path, vscode.window.activeTextEditor.document);
            }
        }
        context.subscriptions.push(vscode.window.onDidChangeActiveTextEditor((e) => {
            if (e !== undefined && (e.document.fileName.endsWith('.brick') || e.document.fileName.endsWith('.openplx'))) {
                loadLocalBundleIfNotInPath(bundle_path, e.document);
            }
        }));
        // The definition provider is used for "Goto definition"
        context.subscriptions.push(vscode.languages.registerDefinitionProvider(selector, definition_provider));
        // The completion item provider handles autocomplete/snippets
        context.subscriptions.push(vscode.languages.registerCompletionItemProvider(selector, new completionitem_provider_1.BrickCompletionItemProvider(brick_context)));
        // The content provider is used for displaying converted imported documents
        if (anytobrick_command) {
            const import_content_provider = new brickimportcontent_provider_1.BrickImportContentProvider(anytobrick_command);
            context.subscriptions.push(vscode.workspace.registerTextDocumentContentProvider('brickimport', import_content_provider));
        }
    }
}
exports.activate = activate;
// This method is called when your extension is deactivated
function deactivate() {
    if (brick_context) {
        brickwasm.destroy_context(brick_context);
    }
    if (!client) {
        return undefined;
    }
    return client.stop();
}
exports.deactivate = deactivate;
//# sourceMappingURL=extension.js.map