"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TokenProvider = void 0;
const vscode = require("vscode");
const path = require("path");
const fs = require("fs");
const brickwasm = require("@algoryx/brickwasm");
class TokenProvider {
    constructor(legend, context, outputChannel, diagnosticCollection) {
        this.legend = legend;
        this.context = context;
        this.outputChannel = outputChannel;
        this.diagnosticCollection = diagnosticCollection;
    }
    provideDocumentSemanticTokens(document, token) {
        const uri = document.uri.scheme === 'file' ? fs.realpathSync.native(path.normalize(document.uri.fsPath)) : document.uri.path;
        const stored = brickwasm.query_document(this.context, uri);
        let namespace = "";
        if (stored.errors.length === 0 && stored.document !== null) {
            namespace = stored.document.namespace.join('.');
        }
        const store_results = brickwasm.store(this.context, document.getText(), uri, namespace);
        const bind_results = brickwasm.bind(this.context);
        const { tokens, errors } = brickwasm.classify_bound_tokens(this.context, uri);
        const all_errors = [...store_results.errors, ...bind_results.errors, ...errors];
        const builder = new vscode.SemanticTokensBuilder(this.legend);
        for (const token of tokens) {
            if (token.type === 'MultiLineString') {
                const lines = token.lexeme.split(/\r?\n/);
                let column = token.column - 1;
                for (let i = 0; i < lines.length; i++) {
                    builder.push(new vscode.Range(new vscode.Position(token.line - 1 + i, column), new vscode.Position(token.line - 1 + i, column + lines[i].length)), 'string');
                    column = 0;
                }
                continue;
            }
            const range = new vscode.Range(new vscode.Position(token.line - 1, token.column - 1), new vscode.Position(token.line - 1, token.column + token.lexeme.length - 1));
            switch (token.classifier) {
                case 'keyword':
                    builder.push(range, 'keyword');
                    break;
                case 'string':
                    if (token.type !== 'MultiLineString') {
                        builder.push(range, 'string');
                    }
                    break;
                case 'number':
                    builder.push(range, 'number');
                    break;
                case 'model':
                    builder.push(range, 'class', ['declaration']);
                    break;
                case 'operator':
                    builder.push(range, 'operator');
                    break;
                case 'var_declaration':
                    builder.push(range, 'variable', ['declaration']);
                    break;
                case 'var_modification':
                    builder.push(range, 'property', ['modification']);
                    break;
                case 'var_reference':
                    builder.push(range, 'property');
                    break;
                case 'type':
                    builder.push(range, 'type');
                    break;
                case 'method_declaration':
                    builder.push(range, 'method', ['declaration']);
                    break;
                case 'parameter_declaration':
                    builder.push(range, 'parameter');
                    break;
            }
        }
        // Convert brick errors to vscode diagnostics.
        const diagnostics = [];
        for (const error of all_errors) {
            if (error.uri !== uri)
                continue;
            // Ignore type not found errors since they don't work with imported files
            if (error.error_code === 'TypeNotFound')
                continue;
            if (error.error_code === 'InternalError') {
                this.outputChannel.appendLine(`brickwasm internal error: ${error.uri}:${error.line}`);
            }
            else {
                const message = `${error.error_code}`;
                diagnostics.push(new vscode.Diagnostic(new vscode.Range(new vscode.Position(error.line - 1, error.column - 1), new vscode.Position(error.line - 1, error.column - 1 + error.width)), message, vscode.DiagnosticSeverity.Error));
            }
        }
        // Publish the errors to the editor
        this.diagnosticCollection.set(document.uri, diagnostics);
        return builder.build();
    }
}
exports.TokenProvider = TokenProvider;
//# sourceMappingURL=token_provider.js.map